﻿using RecrownedAthenaeum.Render;
using RecrownedAthenaeum.Types;
using System;

namespace RecrownedAthenaeum.UI.Modular.Modules
{
    /// <summary>
    /// Represents a texture with more information.
    /// </summary>
    public class Image : UIModule, ISpecialDrawable
    {
        /// <summary>
        /// The rotation of the image.
        /// </summary>
        public float rotation = 0f;

        /// <summary>
        /// The texture to be rendered.
        /// </summary>
        public Texture2D texture;
        
        /// <summary>
        /// Scale of of the X axis.
        /// </summary>
        public float ScaleX { get { return (float)Width / texture.Width; } set { Width = (int)(texture.Width * value); } }

        /// <summary>
        /// Scale of the Y axis.
        /// </summary>
        public float ScaleY { get { return (float)Height / texture.Height; } set { Height = (int)(texture.Height * value); } }

        /// <summary>
        /// Sets scale of X and Y.
        /// </summary>
        public float Scale { set { ScaleY = value; ScaleX = value; } }

        /// <summary>
        /// Constructs an image given a texture.
        /// </summary>
        /// <param name="texture">Texture to use.</param>
        public Image(Texture2D texture)
        {
            this.texture = texture ?? throw new ArgumentException("Image requires a texture.");
            SetPositionAndDimensions(texture.Bounds);
        }

        /// <summary>
        /// Draws the image with default values.
        /// </summary>
        /// <param name="batch">The batch to use.</param>
        public override void Draw(ConsistentSpriteBatch batch)
        {
            batch.Draw(texture, new Rectangle(X, Y, Width, Height), null, color, rotation, origin, SpriteEffects.None, 0f);
            base.Draw(batch);
        }

        /// <summary>
        /// Draws the image with more options.
        /// </summary>
        /// <param name="spriteBatch">Batch used.</param>
        /// <param name="destination">Where to draw texture to.</param>
        /// <param name="color">The color tint to use.</param>
        /// <param name="rotation">Rotation of image.</param>
        /// <param name="origin">Origin for the rotation.</param>
        public void Draw(ConsistentSpriteBatch spriteBatch, Rectangle destination, Color color, float rotation = 0, Vector2 origin = default(Vector2))
        {
            this.color = color;
            this.rotation = rotation;
            this.origin = origin;
            SetPositionAndDimensions(destination);
            Draw(spriteBatch);
        }

        /// <summary>
        /// Center's the origin to the middle of the dimensions of the texture.
        /// </summary>
        public override void CenterOrigin()
        {
            origin.X = texture.Bounds.Width / 2f;
            origin.Y = texture.Bounds.Height / 2f;
        }
    }
}
